import path from 'node:path';
import { File, Request } from '../util/index.js';
import UpdaterVersion from './UpdaterVersion.js';
class AssistantControl {
    async readDataBasePath() {
        const dbPath = process.env.ASSIST_CONTROL_DB_PATH;
        if (!dbPath)
            throw new Error('Variable { ASSIST_CONTROL_DB_PATH } not defined');
        const versionFolders = await File.readdir(dbPath, { onlyType: 'directory' });
        const getVersionByPath = (pathToAnalise) => path.basename(pathToAnalise.replace(path.dirname(pathToAnalise), ''));
        const handleDbFiles = (dbFiles) => {
            if (Array.isArray(dbFiles) && dbFiles.length) {
                return { version: getVersionByPath(dbFiles[0].path), files: dbFiles };
            }
            return {};
        };
        const readDbFiles = async (dir) => {
            const files = await File.readdir(path.join(dir.path, dir.name), { onlyType: 'file' });
            return handleDbFiles(files);
        };
        const databaseFiles = await Promise.all(versionFolders.map(readDbFiles));
        return databaseFiles;
    }
    async getDataBaseList() {
        const dbList = await this.readDataBasePath();
        let response = [];
        dbList.forEach((versionInfo) => {
            if (versionInfo && Array.isArray(versionInfo.files)) {
                response = response.concat(versionInfo.files.map((file) => {
                    file.version = versionInfo.version;
                    return file;
                }));
            }
        });
        return response;
    }
    async downloadDb(response, { version, fileName }) {
        const dbPath = process.env.ASSIST_CONTROL_DB_PATH;
        if (!dbPath) {
            response.status(500).send({
                error: 'Variable { ASSIST_CONTROL_DB_PATH } not defined',
            });
            return;
        }
        response.download(path.join(dbPath, version, fileName), (err) => {
            if (err) {
                console.error(`Erro ao realizar download da versão ${fileName}`, err);
                response.status(500).send({
                    error: `Error ao processar download. Err: ${err.message}`,
                    stack: err.stack,
                    version,
                    fileName,
                });
            }
        });
    }
    getPackageNamesInPage(pageContent, extension = 'tar.gz') {
        const pattern = new RegExp(`(?<=href=")[\\w\\.]+${extension}`, 'gim');
        return pageContent.match(pattern);
    }
    getVersionByPackageName(packageName) {
        const pattern = /[\d\\.]+(?=\.tar\.gz)/i;
        const [result] = packageName.match(pattern) || [];
        return result;
    }
    async loadUpdaterPackages() {
        const packageUrl = Request.removeStartEndBar(process.env.ASSIST_UPDATER_PACKAGE_URL || '');
        const updaterVersionsPath = process.env.UPDATER_VERSIONS_PATH || '';
        if (!packageUrl) {
            return Promise.reject(new Error('Parameter {ASSIST_UPDATER_PACKAGES_URL} not defined'));
        }
        if (!updaterVersionsPath) {
            return Promise.reject(new Error('Parameter {UPDATER_VERSIONS_PATH} not defined'));
        }
        try {
            const response = {
                url: packageUrl,
                success: [],
                fail: [],
            };
            const actualVersions = await UpdaterVersion.list();
            const mappedVersions = actualVersions.map(({ version }) => version);
            const packagePage = await Request.get(packageUrl);
            const packageNames = this.getPackageNamesInPage(packagePage) || [];
            const packagesInServer = packageNames.map((name) => {
                const version = this.getVersionByPackageName(name) || '';
                return {
                    name,
                    version,
                    urlToDownload: `${packageUrl}/${name}`,
                };
            });
            const packagesToAdd = packagesInServer.filter(({ version }) => !mappedVersions.includes(version));
            const promiseCollection = packagesToAdd.map(({ urlToDownload, version }) => {
                const downloadPromise = Request.downloadFile(urlToDownload, path.join(updaterVersionsPath, `${version}.tar.gz`));
                return downloadPromise;
            });
            const downloadResult = await Promise.allSettled(promiseCollection);
            const errors = [];
            const databaseResult = [];
            downloadResult.forEach((result) => {
                if (result.status === 'fulfilled') {
                    const version = this.getVersionByPackageName(result.value.fileName) || '';
                    databaseResult.push(UpdaterVersion.create({ version }));
                }
                else {
                    errors.push(result.reason);
                }
            });
            await Promise.allSettled(databaseResult).then((dbResultResponse) => {
                dbResultResponse.forEach((resp) => {
                    if (resp.status === 'fulfilled') {
                        const savedItem = resp.value;
                        response.success.push(savedItem);
                    }
                    else {
                        response.fail.push({ error: true, message: resp.reason });
                    }
                });
            });
            return response;
        }
        catch (error) {
            return Promise.reject(new Error(`Error in download from ${packageUrl}. Error: ${error.message}`));
        }
    }
}
const instance = new AssistantControl();
export { instance as AssistantControl };
export default instance;
